@@ -1947,11 +1947,16 @@ main (int argc, char **argv)
     { NULL }
   };
 
-  setlocale (LC_ALL, "");
-  textdomain (GETTEXT_PACKAGE);
 #ifdef G_OS_WIN32
   extern gchar *_glib_get_locale_dir (void);
-  gchar *tmp = _glib_get_locale_dir ();
+  gchar *tmp;
+#endif
+
+  setlocale (LC_ALL, "");
+  textdomain (GETTEXT_PACKAGE);
+
+#ifdef G_OS_WIN32
+  tmp = _glib_get_locale_dir ();
   bindtextdomain (GETTEXT_PACKAGE, tmp);
   g_free (tmp);
 #else
