@@ -7,7 +7,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,56 +15,29 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /*
  * MT safe
  */
 
-/**
- * SECTION:queue
- * @Title: Double-ended Queues
- * @Short_description: double-ended queue data structure
- *
- * The #GQueue structure and its associated functions provide a standard
- * queue data structure. Internally, GQueue uses the same data structure
- * as #GList to store elements.
- *
- * The data contained in each element can be either integer values, by
- * using one of the [Type Conversion Macros][glib-Type-Conversion-Macros],
- * or simply pointers to any type of data.
- *
- * As with all other GLib data structures, #GQueue is not thread-safe.
- * For a thread-safe queue, use #GAsyncQueue.
- *
- * To create a new GQueue, use g_queue_new().
- *
- * To initialize a statically-allocated GQueue, use #G_QUEUE_INIT or
- * g_queue_init().
- *
- * To add elements, use g_queue_push_head(), g_queue_push_head_link(),
- * g_queue_push_tail() and g_queue_push_tail_link().
- *
- * To remove elements, use g_queue_pop_head() and g_queue_pop_tail().
- *
- * To free the entire queue, use g_queue_free().
- */
 #include "config.h"
 
 #include "gqueue.h"
 
 #include "gtestutils.h"
-#include "gslice.h"
 
 /**
  * g_queue_new:
  *
  * Creates a new #GQueue.
  *
- * Returns: a newly allocated #GQueue
+ * Returns: a new #GQueue.
  **/
-GQueue *
+GQueue*
 g_queue_new (void)
 {
   return g_slice_new0 (GQueue);
@@ -72,14 +45,11 @@ g_queue_new (void)
 
 /**
  * g_queue_free:
- * @queue: a #GQueue
+ * @queue: a #GQueue.
  *
- * Frees the memory allocated for the #GQueue. Only call this function
- * if @queue was created with g_queue_new(). If queue elements contain
+ * Frees the memory allocated for the #GQueue. Only call this function if
+ * @queue was created with g_queue_new(). If queue elements contain
  * dynamically-allocated memory, they should be freed first.
- *
- * If queue elements contain dynamically-allocated memory, you should
- * either use g_queue_free_full() or free them manually first.
  **/
 void
 g_queue_free (GQueue *queue)
@@ -90,27 +60,6 @@ g_queue_free (GQueue *queue)
   g_slice_free (GQueue, queue);
 }
 
-/**
- * g_queue_free_full:
- * @queue: a pointer to a #GQueue
- * @free_func: the function to be called to free each element's data
- *
- * Convenience method, which frees all the memory used by a #GQueue,
- * and calls the specified destroy function on every element's data.
- *
- * @free_func should not modify the queue (eg, by removing the freed
- * element from it).
- *
- * Since: 2.32
- */
-void
-g_queue_free_full (GQueue        *queue,
-                  GDestroyNotify  free_func)
-{
-  g_queue_foreach (queue, (GFunc) free_func, NULL);
-  g_queue_free (queue);
-}
-
 /**
  * g_queue_init:
  * @queue: an uninitialized #GQueue
@@ -121,7 +70,7 @@ g_queue_free_full (GQueue        *queue,
  * g_queue_new().
  *
  * Since: 2.14
- */
+ **/
 void
 g_queue_init (GQueue *queue)
 {
@@ -155,8 +104,8 @@ g_queue_clear (GQueue *queue)
  *
  * Returns %TRUE if the queue is empty.
  *
- * Returns: %TRUE if the queue is empty
- */
+ * Returns: %TRUE if the queue is empty.
+ **/
 gboolean
 g_queue_is_empty (GQueue *queue)
 {
@@ -171,10 +120,10 @@ g_queue_is_empty (GQueue *queue)
  * 
  * Returns the number of items in @queue.
  * 
- * Returns: the number of items in @queue
+ * Return value: The number of items in @queue.
  * 
  * Since: 2.4
- */
+ **/
 guint
 g_queue_get_length (GQueue *queue)
 {
@@ -190,7 +139,7 @@ g_queue_get_length (GQueue *queue)
  * Reverses the order of the items in @queue.
  * 
  * Since: 2.4
- */
+ **/
 void
 g_queue_reverse (GQueue *queue)
 {
@@ -208,10 +157,10 @@ g_queue_reverse (GQueue *queue)
  * queue consist of pointers to data, the pointers are copied, but the
  * actual data is not.
  * 
- * Returns: a copy of @queue
+ * Return value: A copy of @queue
  * 
  * Since: 2.4
- */
+ **/
 GQueue *
 g_queue_copy (GQueue *queue)
 {
@@ -237,15 +186,12 @@ g_queue_copy (GQueue *queue)
  * Calls @func for each element in the queue passing @user_data to the
  * function.
  * 
- * It is safe for @func to remove the element from @queue, but it must
- * not modify any part of the queue after that element.
- *
  * Since: 2.4
- */
+ **/
 void
 g_queue_foreach (GQueue   *queue,
-                 GFunc     func,
-                 gpointer  user_data)
+		 GFunc     func,
+		 gpointer  user_data)
 {
   GList *list;
 
@@ -268,13 +214,13 @@ g_queue_foreach (GQueue   *queue,
  * 
  * Finds the first link in @queue which contains @data.
  * 
- * Returns: the first link in @queue which contains @data
+ * Return value: The first link in @queue which contains @data.
  * 
  * Since: 2.4
- */
+ **/
 GList *
 g_queue_find (GQueue        *queue,
-              gconstpointer  data)
+	      gconstpointer  data)
 {
   g_return_val_if_fail (queue != NULL, NULL);
 
@@ -286,7 +232,7 @@ g_queue_find (GQueue        *queue,
  * @queue: a #GQueue
  * @data: user data passed to @func
  * @func: a #GCompareFunc to call for each element. It should return 0
- *     when the desired element is found
+ * when the desired element is found
  *
  * Finds an element in a #GQueue, using a supplied function to find the
  * desired element. It iterates over the queue, calling the given function
@@ -294,14 +240,14 @@ g_queue_find (GQueue        *queue,
  * takes two gconstpointer arguments, the #GQueue element's data as the
  * first argument and the given user data as the second argument.
  * 
- * Returns: the found link, or %NULL if it wasn't found
+ * Return value: The found link, or %NULL if it wasn't found
  * 
  * Since: 2.4
- */
+ **/
 GList *
-g_queue_find_custom (GQueue        *queue,
-                     gconstpointer  data,
-                     GCompareFunc   func)
+g_queue_find_custom    (GQueue        *queue,
+			gconstpointer  data,
+			GCompareFunc   func)
 {
   g_return_val_if_fail (queue != NULL, NULL);
   g_return_val_if_fail (func != NULL, NULL);
@@ -321,11 +267,11 @@ g_queue_find_custom (GQueue        *queue,
  * Sorts @queue using @compare_func. 
  * 
  * Since: 2.4
- */
+ **/
 void
 g_queue_sort (GQueue           *queue,
-              GCompareDataFunc  compare_func,
-              gpointer          user_data)
+	      GCompareDataFunc  compare_func,
+	      gpointer          user_data)
 {
   g_return_if_fail (queue != NULL);
   g_return_if_fail (compare_func != NULL);
@@ -340,10 +286,10 @@ g_queue_sort (GQueue           *queue,
  * @data: the data for the new element.
  *
  * Adds a new element at the head of the queue.
- */
+ **/
 void
-g_queue_push_head (GQueue   *queue,
-                   gpointer  data)
+g_queue_push_head (GQueue  *queue,
+		   gpointer data)
 {
   g_return_if_fail (queue != NULL);
 
@@ -361,18 +307,18 @@ g_queue_push_head (GQueue   *queue,
  *     larger than the number of elements in the @queue, the element is
  *     added to the end of the queue.
  * 
- * Inserts a new element into @queue at the given position.
+ * Inserts a new element into @queue at the given position
  * 
  * Since: 2.4
- */
+ **/
 void
 g_queue_push_nth (GQueue   *queue,
-                  gpointer  data,
-                  gint      n)
+		  gpointer  data,
+		  gint      n)
 {
   g_return_if_fail (queue != NULL);
 
-  if (n < 0 || (guint) n >= queue->length)
+  if (n < 0 || n >= queue->length)
     {
       g_queue_push_tail (queue, data);
       return;
@@ -383,14 +329,15 @@ g_queue_push_nth (GQueue   *queue,
 
 /**
  * g_queue_push_head_link:
- * @queue: a #GQueue
- * @link_: a single #GList element, not a list with more than one element
+ * @queue: a #GQueue.
+ * @link_: a single #GList element, <emphasis>not</emphasis> a list with
+ *     more than one element.
  *
  * Adds a new element at the head of the queue.
- */
+ **/
 void
 g_queue_push_head_link (GQueue *queue,
-                        GList  *link)
+			GList  *link)
 {
   g_return_if_fail (queue != NULL);
   g_return_if_fail (link != NULL);
@@ -408,14 +355,14 @@ g_queue_push_head_link (GQueue *queue,
 
 /**
  * g_queue_push_tail:
- * @queue: a #GQueue
- * @data: the data for the new element
+ * @queue: a #GQueue.
+ * @data: the data for the new element.
  *
  * Adds a new element at the tail of the queue.
- */
+ **/
 void
-g_queue_push_tail (GQueue   *queue,
-                   gpointer  data)
+g_queue_push_tail (GQueue  *queue,
+		   gpointer data)
 {
   g_return_if_fail (queue != NULL);
 
@@ -429,14 +376,15 @@ g_queue_push_tail (GQueue   *queue,
 
 /**
  * g_queue_push_tail_link:
- * @queue: a #GQueue
- * @link_: a single #GList element, not a list with more than one element
+ * @queue: a #GQueue.
+ * @link_: a single #GList element, <emphasis>not</emphasis> a list with
+ *   more than one element.
  *
  * Adds a new element at the tail of the queue.
- */
+ **/
 void
 g_queue_push_tail_link (GQueue *queue,
-                        GList  *link)
+			GList  *link)
 {
   g_return_if_fail (queue != NULL);
   g_return_if_fail (link != NULL);
@@ -463,11 +411,11 @@ g_queue_push_tail_link (GQueue *queue,
  * Inserts @link into @queue at the given position.
  * 
  * Since: 2.4
- */
+ **/
 void
-g_queue_push_nth_link (GQueue *queue,
-                       gint    n,
-                       GList  *link_)
+g_queue_push_nth_link  (GQueue  *queue,
+			gint     n,
+			GList   *link_)
 {
   GList *next;
   GList *prev;
@@ -475,7 +423,7 @@ g_queue_push_nth_link (GQueue *queue,
   g_return_if_fail (queue != NULL);
   g_return_if_fail (link_ != NULL);
 
-  if (n < 0 || (guint) n >= queue->length)
+  if (n < 0 || n >= queue->length)
     {
       g_queue_push_tail_link (queue, link_);
       return;
@@ -505,13 +453,13 @@ g_queue_push_nth_link (GQueue *queue,
 
 /**
  * g_queue_pop_head:
- * @queue: a #GQueue
+ * @queue: a #GQueue.
  *
- * Removes the first element of the queue and returns its data.
+ * Removes the first element of the queue.
  *
- * Returns: the data of the first element in the queue, or %NULL
- *     if the queue is empty
- */
+ * Returns: the data of the first element in the queue, or %NULL if the queue
+ *   is empty.
+ **/
 gpointer
 g_queue_pop_head (GQueue *queue)
 {
@@ -524,9 +472,9 @@ g_queue_pop_head (GQueue *queue)
 
       queue->head = node->next;
       if (queue->head)
-        queue->head->prev = NULL;
+	queue->head->prev = NULL;
       else
-        queue->tail = NULL;
+	queue->tail = NULL;
       g_list_free_1 (node);
       queue->length--;
 
@@ -538,14 +486,14 @@ g_queue_pop_head (GQueue *queue)
 
 /**
  * g_queue_pop_head_link:
- * @queue: a #GQueue
+ * @queue: a #GQueue.
  *
- * Removes and returns the first element of the queue.
+ * Removes the first element of the queue.
  *
- * Returns: the #GList element at the head of the queue, or %NULL
- *     if the queue is empty
- */
-GList *
+ * Returns: the #GList element at the head of the queue, or %NULL if the queue
+ *   is empty.
+ **/
+GList*
 g_queue_pop_head_link (GQueue *queue)
 {
   g_return_val_if_fail (queue != NULL, NULL);
@@ -556,12 +504,12 @@ g_queue_pop_head_link (GQueue *queue)
 
       queue->head = node->next;
       if (queue->head)
-        {
-          queue->head->prev = NULL;
-          node->next = NULL;
-        }
+	{
+	  queue->head->prev = NULL;
+	  node->next = NULL;
+	}
       else
-        queue->tail = NULL;
+	queue->tail = NULL;
       queue->length--;
 
       return node;
@@ -574,13 +522,13 @@ g_queue_pop_head_link (GQueue *queue)
  * g_queue_peek_head_link:
  * @queue: a #GQueue
  * 
- * Returns the first link in @queue.
+ * Returns the first link in @queue
  * 
- * Returns: the first link in @queue, or %NULL if @queue is empty
+ * Return value: the first link in @queue, or %NULL if @queue is empty
  * 
  * Since: 2.4
- */
-GList *
+ **/
+GList*
 g_queue_peek_head_link (GQueue *queue)
 {
   g_return_val_if_fail (queue != NULL, NULL);
@@ -592,13 +540,13 @@ g_queue_peek_head_link (GQueue *queue)
  * g_queue_peek_tail_link:
  * @queue: a #GQueue
  * 
- * Returns the last link in @queue.
+ * Returns the last link @queue.
  * 
- * Returns: the last link in @queue, or %NULL if @queue is empty
+ * Return value: the last link in @queue, or %NULL if @queue is empty
  * 
  * Since: 2.4
- */
-GList *
+ **/
+GList*
 g_queue_peek_tail_link (GQueue *queue)
 {
   g_return_val_if_fail (queue != NULL, NULL);
@@ -608,13 +556,13 @@ g_queue_peek_tail_link (GQueue *queue)
 
 /**
  * g_queue_pop_tail:
- * @queue: a #GQueue
+ * @queue: a #GQueue.
  *
- * Removes the last element of the queue and returns its data.
+ * Removes the last element of the queue.
  *
- * Returns: the data of the last element in the queue, or %NULL
- *     if the queue is empty
- */
+ * Returns: the data of the last element in the queue, or %NULL if the queue
+ *   is empty.
+ **/
 gpointer
 g_queue_pop_tail (GQueue *queue)
 {
@@ -627,9 +575,9 @@ g_queue_pop_tail (GQueue *queue)
 
       queue->tail = node->prev;
       if (queue->tail)
-        queue->tail->next = NULL;
+	queue->tail->next = NULL;
       else
-        queue->head = NULL;
+	queue->head = NULL;
       queue->length--;
       g_list_free_1 (node);
 
@@ -642,17 +590,17 @@ g_queue_pop_tail (GQueue *queue)
 /**
  * g_queue_pop_nth:
  * @queue: a #GQueue
- * @n: the position of the element
+ * @n: the position of the element.
  * 
- * Removes the @n'th element of @queue and returns its data.
+ * Removes the @n'th element of @queue.
  * 
- * Returns: the element's data, or %NULL if @n is off the end of @queue
+ * Return value: the element's data, or %NULL if @n is off the end of @queue.
  * 
  * Since: 2.4
- */
+ **/
 gpointer
 g_queue_pop_nth (GQueue *queue,
-                 guint   n)
+		 guint   n)
 {
   GList *nth_link;
   gpointer result;
@@ -672,14 +620,14 @@ g_queue_pop_nth (GQueue *queue,
 
 /**
  * g_queue_pop_tail_link:
- * @queue: a #GQueue
+ * @queue: a #GQueue.
  *
- * Removes and returns the last element of the queue.
+ * Removes the last element of the queue.
  *
- * Returns: the #GList element at the tail of the queue, or %NULL
- *     if the queue is empty
- */
-GList *
+ * Returns: the #GList element at the tail of the queue, or %NULL if the queue
+ *   is empty.
+ **/
+GList*
 g_queue_pop_tail_link (GQueue *queue)
 {
   g_return_val_if_fail (queue != NULL, NULL);
@@ -690,12 +638,12 @@ g_queue_pop_tail_link (GQueue *queue)
       
       queue->tail = node->prev;
       if (queue->tail)
-        {
-          queue->tail->next = NULL;
-          node->prev = NULL;
-        }
+	{
+	  queue->tail->next = NULL;
+	  node->prev = NULL;
+	}
       else
-        queue->head = NULL;
+	queue->head = NULL;
       queue->length--;
       
       return node;
@@ -711,13 +659,13 @@ g_queue_pop_tail_link (GQueue *queue)
  * 
  * Removes and returns the link at the given position.
  * 
- * Returns: the @n'th link, or %NULL if @n is off the end of @queue
+ * Return value: The @n'th link, or %NULL if @n is off the end of @queue.
  * 
  * Since: 2.4
- */
+ **/
 GList*
 g_queue_pop_nth_link (GQueue *queue,
-                      guint   n)
+		      guint   n)
 {
   GList *link;
   
@@ -739,17 +687,17 @@ g_queue_pop_nth_link (GQueue *queue,
  * 
  * Returns the link at the given position
  * 
- * Returns: the link at the @n'th position, or %NULL
- *     if @n is off the end of the list
+ * Return value: The link at the @n'th position, or %NULL if @n is off the
+ * end of the list
  * 
  * Since: 2.4
- */
+ **/
 GList *
 g_queue_peek_nth_link (GQueue *queue,
-                       guint   n)
+		       guint   n)
 {
   GList *link;
-  guint i;
+  gint i;
   
   g_return_val_if_fail (queue != NULL, NULL);
 
@@ -762,13 +710,13 @@ g_queue_peek_nth_link (GQueue *queue,
 
       link = queue->tail;
       for (i = 0; i < n; ++i)
-        link = link->prev;
+	link = link->prev;
     }
   else
     {
       link = queue->head;
       for (i = 0; i < n; ++i)
-        link = link->next;
+	link = link->next;
     }
 
   return link;
@@ -777,18 +725,18 @@ g_queue_peek_nth_link (GQueue *queue,
 /**
  * g_queue_link_index:
  * @queue: a #GQueue
- * @link_: a #GList link
+ * @link_: A #GList link
  * 
  * Returns the position of @link_ in @queue.
  * 
- * Returns: the position of @link_, or -1 if the link is
- *     not part of @queue
+ * Return value: The position of @link_, or -1 if the link is
+ * not part of @queue
  * 
  * Since: 2.4
- */
+ **/
 gint
 g_queue_link_index (GQueue *queue,
-                    GList  *link_)
+		    GList  *link_)
 {
   g_return_val_if_fail (queue != NULL, -1);
 
@@ -796,20 +744,20 @@ g_queue_link_index (GQueue *queue,
 }
 
 /**
- * g_queue_unlink:
+ * g_queue_unlink
  * @queue: a #GQueue
- * @link_: a #GList link that must be part of @queue
+ * @link_: a #GList link that <emphasis>must</emphasis> be part of @queue
  *
- * Unlinks @link_ so that it will no longer be part of @queue.
- * The link is not freed.
+ * Unlinks @link_ so that it will no longer be part of @queue. The link is
+ * not freed.
  *
- * @link_ must be part of @queue.
+ * @link_ must be part of @queue,
  * 
  * Since: 2.4
- */
+ **/
 void
 g_queue_unlink (GQueue *queue,
-                GList  *link_)
+		GList  *link_)
 {
   g_return_if_fail (queue != NULL);
   g_return_if_fail (link_ != NULL);
@@ -824,17 +772,17 @@ g_queue_unlink (GQueue *queue,
 /**
  * g_queue_delete_link:
  * @queue: a #GQueue
- * @link_: a #GList link that must be part of @queue
+ * @link_: a #GList link that <emphasis>must</emphasis> be part of @queue
  *
  * Removes @link_ from @queue and frees it.
  *
  * @link_ must be part of @queue.
  * 
  * Since: 2.4
- */
+ **/
 void
 g_queue_delete_link (GQueue *queue,
-                     GList  *link_)
+		     GList  *link_)
 {
   g_return_if_fail (queue != NULL);
   g_return_if_fail (link_ != NULL);
@@ -845,13 +793,13 @@ g_queue_delete_link (GQueue *queue,
 
 /**
  * g_queue_peek_head:
- * @queue: a #GQueue
+ * @queue: a #GQueue.
  *
  * Returns the first element of the queue.
  *
- * Returns: the data of the first element in the queue, or %NULL
- *     if the queue is empty
- */
+ * Returns: the data of the first element in the queue, or %NULL if the queue
+ *   is empty.
+ **/
 gpointer
 g_queue_peek_head (GQueue *queue)
 {
@@ -862,13 +810,13 @@ g_queue_peek_head (GQueue *queue)
 
 /**
  * g_queue_peek_tail:
- * @queue: a #GQueue
+ * @queue: a #GQueue.
  *
  * Returns the last element of the queue.
  *
- * Returns: the data of the last element in the queue, or %NULL
- *     if the queue is empty
- */
+ * Returns: the data of the last element in the queue, or %NULL if the queue
+ *   is empty.
+ **/
 gpointer
 g_queue_peek_tail (GQueue *queue)
 {
@@ -880,18 +828,18 @@ g_queue_peek_tail (GQueue *queue)
 /**
  * g_queue_peek_nth:
  * @queue: a #GQueue
- * @n: the position of the element
+ * @n: the position of the element.
  * 
  * Returns the @n'th element of @queue. 
  * 
- * Returns: the data for the @n'th element of @queue,
- *     or %NULL if @n is off the end of @queue
+ * Return value: The data for the @n'th element of @queue, or %NULL if @n is
+ *   off the end of @queue.
  * 
  * Since: 2.4
- */
+ **/
 gpointer
 g_queue_peek_nth (GQueue *queue,
-                  guint   n)
+		  guint   n)
 {
   GList *link;
   
@@ -908,18 +856,17 @@ g_queue_peek_nth (GQueue *queue,
 /**
  * g_queue_index:
  * @queue: a #GQueue
- * @data: the data to find
+ * @data: the data to find.
  * 
  * Returns the position of the first element in @queue which contains @data.
  * 
- * Returns: the position of the first element in @queue which
- *     contains @data, or -1 if no element in @queue contains @data
+ * Return value: The position of the first element in @queue which contains @data, or -1 if no element in @queue contains @data.
  * 
  * Since: 2.4
- */
+ **/
 gint
 g_queue_index (GQueue        *queue,
-               gconstpointer  data)
+	       gconstpointer  data)
 {
   g_return_val_if_fail (queue != NULL, -1);
 
@@ -929,17 +876,17 @@ g_queue_index (GQueue        *queue,
 /**
  * g_queue_remove:
  * @queue: a #GQueue
- * @data: the data to remove
+ * @data: data to remove.
  * 
  * Removes the first element in @queue that contains @data. 
  * 
- * Returns: %TRUE if @data was found and removed from @queue
+ * Return value: %TRUE if @data was found and removed from @queue
  *
  * Since: 2.4
- */
+ **/
 gboolean
 g_queue_remove (GQueue        *queue,
-                gconstpointer  data)
+		gconstpointer  data)
 {
   GList *link;
   
@@ -956,17 +903,17 @@ g_queue_remove (GQueue        *queue,
 /**
  * g_queue_remove_all:
  * @queue: a #GQueue
- * @data: the data to remove
+ * @data: data to remove
  * 
  * Remove all elements whose data equals @data from @queue.
  * 
- * Returns: the number of elements removed from @queue
+ * Return value: the number of elements removed from @queue
  *
  * Since: 2.4
- */
+ **/
 guint
 g_queue_remove_all (GQueue        *queue,
-                    gconstpointer  data)
+		    gconstpointer  data)
 {
   GList *list;
   guint old_length;
@@ -981,7 +928,7 @@ g_queue_remove_all (GQueue        *queue,
       GList *next = list->next;
 
       if (list->data == data)
-        g_queue_delete_link (queue, list);
+	g_queue_delete_link (queue, list);
       
       list = next;
     }
@@ -992,62 +939,49 @@ g_queue_remove_all (GQueue        *queue,
 /**
  * g_queue_insert_before:
  * @queue: a #GQueue
- * @sibling: (nullable): a #GList link that must be part of @queue, or %NULL to
- *   push at the tail of the queue.
+ * @sibling: a #GList link that <emphasis>must</emphasis> be part of @queue
  * @data: the data to insert
  * 
  * Inserts @data into @queue before @sibling.
  *
- * @sibling must be part of @queue. Since GLib 2.44 a %NULL sibling pushes the
- * data at the tail of the queue.
+ * @sibling must be part of @queue.
  * 
  * Since: 2.4
- */
+ **/
 void
 g_queue_insert_before (GQueue   *queue,
-                       GList    *sibling,
-                       gpointer  data)
+		       GList    *sibling,
+		       gpointer  data)
 {
   g_return_if_fail (queue != NULL);
+  g_return_if_fail (sibling != NULL);
 
-  if (sibling == NULL)
-    {
-      /* We don't use g_list_insert_before() with a NULL sibling because it
-       * would be a O(n) operation and we would need to update manually the tail
-       * pointer.
-       */
-      g_queue_push_tail (queue, data);
-    }
-  else
-    {
-      queue->head = g_list_insert_before (queue->head, sibling, data);
-      queue->length++;
-    }
+  queue->head = g_list_insert_before (queue->head, sibling, data);
+  queue->length++;
 }
 
 /**
  * g_queue_insert_after:
  * @queue: a #GQueue
- * @sibling: (nullable): a #GList link that must be part of @queue, or %NULL to
- *   push at the head of the queue.
+ * @sibling: a #GList link that <emphasis>must</emphasis> be part of @queue
  * @data: the data to insert
  *
- * Inserts @data into @queue after @sibling.
+ * Inserts @data into @queue after @sibling
  *
- * @sibling must be part of @queue. Since GLib 2.44 a %NULL sibling pushes the
- * data at the head of the queue.
+ * @sibling must be part of @queue
  * 
  * Since: 2.4
- */
+ **/
 void
 g_queue_insert_after (GQueue   *queue,
-                      GList    *sibling,
-                      gpointer  data)
+		      GList    *sibling,
+		      gpointer  data)
 {
   g_return_if_fail (queue != NULL);
+  g_return_if_fail (sibling != NULL);
 
-  if (sibling == NULL)
-    g_queue_push_head (queue, data);
+  if (sibling == queue->tail)
+    g_queue_push_tail (queue, data);
   else
     g_queue_insert_before (queue, sibling->next, data);
 }
@@ -1061,17 +995,17 @@ g_queue_insert_after (GQueue   *queue,
  *     return 0 if the elements are equal, a negative value if the first
  *     element comes before the second, and a positive value if the second
  *     element comes before the first.
- * @user_data: user data passed to @func
+ * @user_data: user data passed to @func.
  * 
  * Inserts @data into @queue using @func to determine the new position.
  * 
  * Since: 2.4
- */
+ **/
 void
 g_queue_insert_sorted (GQueue           *queue,
-                       gpointer          data,
-                       GCompareDataFunc  func,
-                       gpointer          user_data)
+		       gpointer          data,
+		       GCompareDataFunc  func,
+		       gpointer          user_data)
 {
   GList *list;
   
@@ -1081,5 +1015,8 @@ g_queue_insert_sorted (GQueue           *queue,
   while (list && func (list->data, data, user_data) < 0)
     list = list->next;
 
-  g_queue_insert_before (queue, list, data);
+  if (list)
+    g_queue_insert_before (queue, list, data);
+  else
+    g_queue_push_tail (queue, data);
 }
