@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,21 +12,20 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General
- * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "config.h"
 
 #include <string.h>
 
-/* for GValueArray */
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 #include "gboxed.h"
-#include "gclosure.h"
 #include "gtype-private.h"
 #include "gvalue.h"
 #include "gvaluearray.h"
+#include "gclosure.h"
 #include "gvaluecollector.h"
 
 
@@ -37,24 +36,13 @@
  * @see_also: #GParamSpecBoxed, g_param_spec_boxed()
  * @title: Boxed Types
  *
- * #GBoxed is a generic wrapper mechanism for arbitrary C structures. The only
- * thing the type system needs to know about the structures is how to copy them
- * (a #GBoxedCopyFunc) and how to free them (a #GBoxedFreeFunc) — beyond that
- * they are treated as opaque chunks of memory.
+ * GBoxed is a generic wrapper mechanism for arbitrary C structures. The only
+ * thing the type system needs to know about the structures is how to copy and
+ * free them, beyond that they are treated as opaque chunks of memory.
  *
  * Boxed types are useful for simple value-holder structures like rectangles or
- * points. They can also be used for wrapping structures defined in non-#GObject
- * based libraries. They allow arbitrary structures to be handled in a uniform
- * way, allowing uniform copying (or referencing) and freeing (or unreferencing)
- * of them, and uniform representation of the type of the contained structure.
- * In turn, this allows any type which can be boxed to be set as the data in a
- * #GValue, which allows for polymorphic handling of a much wider range of data
- * types, and hence usage of such types as #GObject property values.
- *
- * #GBoxed is designed so that reference counted types can be boxed. Use the
- * type’s ‘ref’ function as the #GBoxedCopyFunc, and its ‘unref’ function as the
- * #GBoxedFreeFunc. For example, for #GBytes, the #GBoxedCopyFunc is
- * g_bytes_ref(), and the #GBoxedFreeFunc is g_bytes_unref().
+ * points. They can also be used for wrapping structures defined in non-GObject
+ * based libraries.
  */
 
 static inline void              /* keep this function in sync with gvalue.c */
@@ -86,19 +74,10 @@ value_free (GValue *value)
   g_free (value);
 }
 
-static GPollFD *
-pollfd_copy (GPollFD *src)
-{
-  GPollFD *dest = g_new0 (GPollFD, 1);
-  /* just a couple of integers */
-  memcpy (dest, src, sizeof (GPollFD));
-  return dest;
-}
-
 void
 _g_boxed_type_init (void)
 {
-  const GTypeInfo info = {
+  static const GTypeInfo info = {
     0,                          /* class_size */
     NULL,                       /* base_init */
     NULL,                       /* base_destroy */
@@ -111,7 +90,7 @@ _g_boxed_type_init (void)
     NULL,                       /* value_table */
   };
   const GTypeFundamentalInfo finfo = { G_TYPE_FLAG_DERIVABLE, };
-  GType type G_GNUC_UNUSED  /* when compiling with G_DISABLE_ASSERT */;
+  GType type;
 
   /* G_TYPE_BOXED
    */
@@ -120,6 +99,12 @@ _g_boxed_type_init (void)
   g_assert (type == G_TYPE_BOXED);
 }
 
+static GDate *
+gdate_copy (GDate *date)
+{
+  return g_date_new_julian (g_date_get_julian (date));
+}
+
 static GString *
 gstring_copy (GString *src_gstring)
 {
@@ -135,42 +120,29 @@ gstring_free (GString *gstring)
 G_DEFINE_BOXED_TYPE (GClosure, g_closure, g_closure_ref, g_closure_unref)
 G_DEFINE_BOXED_TYPE (GValue, g_value, value_copy, value_free)
 G_DEFINE_BOXED_TYPE (GValueArray, g_value_array, g_value_array_copy, g_value_array_free)
-G_DEFINE_BOXED_TYPE (GDate, g_date, g_date_copy, g_date_free)
+G_DEFINE_BOXED_TYPE (GDate, g_date, gdate_copy, g_date_free)
 /* the naming is a bit odd, but GString is obviously not G_TYPE_STRING */
 G_DEFINE_BOXED_TYPE (GString, g_gstring, gstring_copy, gstring_free)
 G_DEFINE_BOXED_TYPE (GHashTable, g_hash_table, g_hash_table_ref, g_hash_table_unref)
 G_DEFINE_BOXED_TYPE (GArray, g_array, g_array_ref, g_array_unref)
 G_DEFINE_BOXED_TYPE (GPtrArray, g_ptr_array,g_ptr_array_ref, g_ptr_array_unref)
 G_DEFINE_BOXED_TYPE (GByteArray, g_byte_array, g_byte_array_ref, g_byte_array_unref)
-G_DEFINE_BOXED_TYPE (GBytes, g_bytes, g_bytes_ref, g_bytes_unref)
 
+#ifdef ENABLE_REGEX
 G_DEFINE_BOXED_TYPE (GRegex, g_regex, g_regex_ref, g_regex_unref)
-G_DEFINE_BOXED_TYPE (GMatchInfo, g_match_info, g_match_info_ref, g_match_info_unref)
+#else
+GType g_regex_get_type (void) { return G_TYPE_INVALID; }
+#endif /* ENABLE_REGEX */
 
 #define g_variant_type_get_type g_variant_type_get_gtype
 G_DEFINE_BOXED_TYPE (GVariantType, g_variant_type, g_variant_type_copy, g_variant_type_free)
 #undef g_variant_type_get_type
 
 G_DEFINE_BOXED_TYPE (GVariantBuilder, g_variant_builder, g_variant_builder_ref, g_variant_builder_unref)
-G_DEFINE_BOXED_TYPE (GVariantDict, g_variant_dict, g_variant_dict_ref, g_variant_dict_unref)
 
 G_DEFINE_BOXED_TYPE (GError, g_error, g_error_copy, g_error_free)
 
-G_DEFINE_BOXED_TYPE (GDateTime, g_date_time, g_date_time_ref, g_date_time_unref)
-G_DEFINE_BOXED_TYPE (GTimeZone, g_time_zone, g_time_zone_ref, g_time_zone_unref)
-G_DEFINE_BOXED_TYPE (GKeyFile, g_key_file, g_key_file_ref, g_key_file_unref)
-G_DEFINE_BOXED_TYPE (GMappedFile, g_mapped_file, g_mapped_file_ref, g_mapped_file_unref)
-
-G_DEFINE_BOXED_TYPE (GMainLoop, g_main_loop, g_main_loop_ref, g_main_loop_unref)
-G_DEFINE_BOXED_TYPE (GMainContext, g_main_context, g_main_context_ref, g_main_context_unref)
-G_DEFINE_BOXED_TYPE (GSource, g_source, g_source_ref, g_source_unref)
-G_DEFINE_BOXED_TYPE (GPollFD, g_pollfd, pollfd_copy, g_free)
-G_DEFINE_BOXED_TYPE (GMarkupParseContext, g_markup_parse_context, g_markup_parse_context_ref, g_markup_parse_context_unref)
-
-G_DEFINE_BOXED_TYPE (GThread, g_thread, g_thread_ref, g_thread_unref)
-G_DEFINE_BOXED_TYPE (GChecksum, g_checksum, g_checksum_copy, g_checksum_free)
-
-G_DEFINE_BOXED_TYPE (GOptionGroup, g_option_group, g_option_group_ref, g_option_group_unref)
+G_DEFINE_BOXED_TYPE (GDateTime, g_date_time, g_date_time_ref, g_date_time_unref);
 
 /* This one can't use G_DEFINE_BOXED_TYPE (GStrv, g_strv, g_strdupv, g_strfreev) */
 GType
@@ -191,6 +163,12 @@ g_strv_get_type (void)
   return g_define_type_id__volatile;
 }
 
+/**
+ * g_variant_get_gtype:
+ *
+ * Since: 2.24
+ * Deprecated: 2.26
+ */
 GType
 g_variant_get_gtype (void)
 {
@@ -257,7 +235,7 @@ boxed_proxy_lcopy_value (const GValue *value,
   gpointer *boxed_p = collect_values[0].v_pointer;
 
   if (!boxed_p)
-    return g_strdup_printf ("value location for '%s' passed as NULL", G_VALUE_TYPE_NAME (value));
+    return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
 
   if (!value->data[0].v_pointer)
     *boxed_p = NULL;
@@ -317,7 +295,7 @@ g_boxed_type_register_static (const gchar   *name,
 
   type = g_type_register_static (G_TYPE_BOXED, name, &type_info, 0);
 
-  /* install proxy functions upon successful registration */
+  /* install proxy functions upon successfull registration */
   if (type)
     _g_type_boxed_init (type, boxed_copy, boxed_free);
 
@@ -327,12 +305,11 @@ g_boxed_type_register_static (const gchar   *name,
 /**
  * g_boxed_copy:
  * @boxed_type: The type of @src_boxed.
- * @src_boxed: (not nullable): The boxed structure to be copied.
+ * @src_boxed: The boxed structure to be copied.
  * 
  * Provide a copy of a boxed structure @src_boxed which is of type @boxed_type.
  * 
- * Returns: (transfer full) (not nullable): The newly created copy of the boxed
- *    structure.
+ * Returns: The newly created copy of the boxed structure.
  */
 gpointer
 g_boxed_copy (GType         boxed_type,
@@ -346,7 +323,8 @@ g_boxed_copy (GType         boxed_type,
   g_return_val_if_fail (src_boxed != NULL, NULL);
 
   value_table = g_type_value_table_peek (boxed_type);
-  g_assert (value_table != NULL);
+  if (!value_table)
+    g_return_val_if_fail (G_TYPE_IS_VALUE_TYPE (boxed_type), NULL);
 
   /* check if our proxying implementation is used, we can short-cut here */
   if (value_table->value_copy == boxed_proxy_value_copy)
@@ -360,7 +338,7 @@ g_boxed_copy (GType         boxed_type,
        * (data[0].v_pointer is the boxed struct, and
        * data[1].v_uint holds the G_VALUE_NOCOPY_CONTENTS flag,
        * rest zero).
-       * but then, we can expect that since we laid out the
+       * but then, we can expect that since we layed out the
        * g_boxed_*() API.
        * data[1].v_uint&G_VALUE_NOCOPY_CONTENTS shouldn't be set
        * after a copy.
@@ -376,7 +354,7 @@ g_boxed_copy (GType         boxed_type,
 
       /* double check and grouse if things went wrong */
       if (dest_value.data[1].v_ulong)
-	g_warning ("the copy_value() implementation of type '%s' seems to make use of reserved GValue fields",
+	g_warning ("the copy_value() implementation of type `%s' seems to make use of reserved GValue fields",
 		   g_type_name (boxed_type));
 
       dest_boxed = dest_value.data[0].v_pointer;
@@ -388,7 +366,7 @@ g_boxed_copy (GType         boxed_type,
 /**
  * g_boxed_free:
  * @boxed_type: The type of @boxed.
- * @boxed: (not nullable): The boxed structure to be freed.
+ * @boxed: The boxed structure to be freed.
  *
  * Free the boxed structure @boxed which is of type @boxed_type.
  */
@@ -403,7 +381,8 @@ g_boxed_free (GType    boxed_type,
   g_return_if_fail (boxed != NULL);
 
   value_table = g_type_value_table_peek (boxed_type);
-  g_assert (value_table != NULL);
+  if (!value_table)
+    g_return_if_fail (G_TYPE_IS_VALUE_TYPE (boxed_type));
 
   /* check if our proxying implementation is used, we can short-cut here */
   if (value_table->value_free == boxed_proxy_value_free)
@@ -482,7 +461,7 @@ value_set_boxed_internal (GValue       *value,
 /**
  * g_value_set_boxed:
  * @value: a valid #GValue of %G_TYPE_BOXED derived type
- * @v_boxed: (nullable): boxed value to be set
+ * @v_boxed: boxed value to be set
  *
  * Set the contents of a %G_TYPE_BOXED derived #GValue to @v_boxed.
  */
@@ -499,7 +478,7 @@ g_value_set_boxed (GValue       *value,
 /**
  * g_value_set_static_boxed:
  * @value: a valid #GValue of %G_TYPE_BOXED derived type
- * @v_boxed: (nullable): static boxed value to be set
+ * @v_boxed: static boxed value to be set
  *
  * Set the contents of a %G_TYPE_BOXED derived #GValue to @v_boxed.
  * The boxed value is assumed to be static, and is thus not duplicated
@@ -518,7 +497,7 @@ g_value_set_static_boxed (GValue       *value,
 /**
  * g_value_set_boxed_take_ownership:
  * @value: a valid #GValue of %G_TYPE_BOXED derived type
- * @v_boxed: (nullable): duplicated unowned boxed value to be set
+ * @v_boxed: duplicated unowned boxed value to be set
  *
  * This is an internal function introduced mainly for C marshallers.
  *
@@ -534,11 +513,11 @@ g_value_set_boxed_take_ownership (GValue       *value,
 /**
  * g_value_take_boxed:
  * @value: a valid #GValue of %G_TYPE_BOXED derived type
- * @v_boxed: (nullable): duplicated unowned boxed value to be set
+ * @v_boxed: duplicated unowned boxed value to be set
  *
  * Sets the contents of a %G_TYPE_BOXED derived #GValue to @v_boxed
- * and takes over the ownership of the caller’s reference to @v_boxed;
- * the caller doesn’t have to unref it any more.
+ * and takes over the ownership of the callers reference to @v_boxed;
+ * the caller doesn't have to unref it any more.
  *
  * Since: 2.4
  */
